"""async function logMovies() {
  const response = await fetch("http://example.com/movies.json");
  const movies = await response.json();
  console.log(movies);
}"""


class fetch:
    def __init__(self):
        pass

    def get(self, url):
        pass

    def post(self, url):
        pass

    def put(self, url):
        pass

    def patch(self, url):
        pass

    def delete(self, url):
        pass




class Style:
    def __init__(self, selector):
        self.selector = selector
        self.properties = {}

    def add_property(self, property_name, value):
        self.properties[property_name] = value

    def __str__(self):
        # Construct the CSS string for the selector and its properties
        css = f"{self.selector} {{\n"
        for prop, value in self.properties.items():
            css += f"    {prop.replace('_', '-')} : {value};\n"
        css += "}\n"
        return css


# external style

class Css:
    def __init__(self, style, fileName=None):
        self.style = style
        self.fileName = fileName

    # def render(self):
    #     pass

    def render(self):
        if self.fileName:
            with open(f'./output/{self.fileName}.css', 'w') as file:
                file.write(self.style)
            link_tag = f'<link rel="stylesheet" type="text/css" href="{self.fileName}.css">'
            return link_tag
        elif self.style:
            style_tag = f'<style>{self.style}</style>'
            return style_tag
        else:
            return ''

    def __str__(self):
        return self.style

    def remove(self):
        pass



# Static Site Generation

import requests

try:
    res = requests.get("https://jsonplaceholder.typicode.com/todos")
    data = res.json()
    print(data[2]["title"])

except requests.exceptions.RequestException as e:
    print("Error:", e)


# form

class Form:
    def __init__(self):
        self.fields = []
        self.action = ''
        self.method = ''

    def addField(self, field):
        self.fields.append(str(field))

    def addFields(self, *fields):
        for field in fields:
            self.fields.append(str(field))

    def __str__(self):
        form = f'<form action="{self.action}" method="{self.method}">\n'
        for field in self.fields:
            form += str(field) + "\n"
        form += "</form>\n"
        return form


class Label:
    def __init__(self, text, for_i=None):
        self.text = text
        self.for_input = for_i

    def __str__(self):
        if '\n' in self.text:
            text_with_br = self.text.replace('\n', '')
            return f'<label for="{self.for_input}">{text_with_br}</label><br>\n'
        elif self.for_input:
            return f'<label for="{self.for_input}">{self.text}</label>\n'
        else:
            return f'<label>{self.text}</label>\n'


class InputField:
    def __init__(self, name, **attributes):
        self.name = name
        self.type = "text"
        self.label = None
        self.attributes = attributes

    def __str__(self):
        input_ = ""  # Define input_ variable here
        attribute_string = ' '.join([f'{key}="{value}"' for key, value in self.attributes.items()])

        if 'className' in self.attributes:
            attribute_string = attribute_string.replace('className', 'class')

        if '\n' in self.name:
            self.name = self.name.replace('\n', '')
            input_ = f'<input type="{self.type}" name="{self.name}"><br>\n'

        else:
            input_ = f'<input type="{self.type}" name="{self.name}">\n'

        if self.label:
            input_ = str(self.label) + input_

        elif attribute_string:
            input_ = f'<input type="{self.type}" name="{self.name} {attribute_string}">'

        return input_



import os

from GooBa.hmr.hmr import javascript


class Document:
    def __init__(self):
        self.head = ""
        self.body_ = ""
        self.styles = []
        self.extern_js = []

    def title(self, text):
        self.head += f"<title>{text}</title>\n"

    def meta(self, **attributes):
        meta_tag = "<meta "
        for key, value in attributes.items():
            meta_tag += f'{key}="{value}" '
        meta_tag += "/>\n"
        self.head += meta_tag

    def add_style(self, *style):
        self.styles.extend(style)

    def body(self, *elements: object) -> object:
        modified_elements = '\n'.join([str(element) for element in elements])
        self.body_ = f"{modified_elements}"

    def add_Head(self, newHead):
        self.head += newHead + "\n"

    def add_EternalJs(self, extern_JS):
        if extern_JS:
            self.extern_js.append(extern_JS)

    def build(self):
        styles_str = '\n'.join(map(str, self.styles))
        external_js_str = '\n'.join(f'<script src="{path}.js"></script>' for path in self.extern_js)

        html_content = f"""<!DOCTYPE html>
        <html>
        <head>
        <style>
        {styles_str}
        </style>
        {self.head}
        </head>
        <body>
        {self.body_}
        </body>
        <script src="./hmr.js"></script>
        <script src="./router.js"></script>
        {external_js_str}
        </html>
        """
        if not os.path.exists('./output'):
            os.makedirs('./output')

        with open('./output/index.html', 'w') as file:
            file.write(html_content)
            print(html_content)

        with open('./output/hmr.js', 'w') as file:
            file.write(javascript)



class Rendering:
    def __init__(self, url: str, selectors: dict):
        self.url = url
        self.selectors = selectors

    def generate_fetch_code(self):
        fetch_code = ""
        for key, selector in self.selectors.items():
            fetch_code += f"""
                document.querySelector("{selector}").innerText = data.{key};\n
            """
        return fetch_code

    def render(self):
        js_code = f"""
            async function fetchData() {{
                try {{
                    const response = await fetch('{self.url}');
                    if (!response.ok) {{
                        throw new Error('Network response was not ok');
                    }}
                    const data = await response.json();
                    {self.generate_fetch_code()}
                }} catch (error) {{
                    console.error('Error:', error);
                }}
            }}

            document.addEventListener("DOMContentLoaded", function() {{
                fetchData();
            }});
        """
        with open("./output/CSR.js", 'w') as file:
            file.write(js_code)



class Element:
    def __init__(self, tag, **attributes):
        self.tag = tag
        self.attributes = attributes
        self.text = None

    def __str__(self):
        attribute_string = ' '.join([f'{key}="{value}"' for key, value in self.attributes.items()])

        if 'className' in self.attributes:
            attribute_string = attribute_string.replace('className', 'class')

        if self.text is not None:
            return f'<{self.tag} {attribute_string}>{self.text}</{self.tag}>'
        else:
            return f'<{self.tag} {attribute_string}></{self.tag}>'

    def __add__(self, other):
        if isinstance(other, Element):
            return self.__str__() + other.__str__()
        else:
            return self.__str__() + str(other)


class Link(Element):
    def __init__(self, href, **attributes):
        super().__init__('a', **attributes)
        self.attributes['href'] = href



routes = {
    "/": "<h1>Home Page</h1>",
    "/about": "<h1>About Page</h1>",
    "/contact": "<h1>Contact Page</h1>"
}

print(f'Routes are {routes}')

Based = f"""


function handleRoute() {{
    const path = window.location.pathname;
    switch (path) {{
        case '/about':
            renderContent("<h1>About Page</h1>");
            break;
        case '/contact':
            renderContent("<h1>Contact Page</h1>");
            break;
        default:
            renderContent("<h1>Home Page</h1>");
            break;
    }}
}}

function renderContent(content) {{
    const root = document.getElementById("root");
    if (root) {{
        root.innerHTML = content;
    }}
}}

document.addEventListener('DOMContentLoaded', () => {{
    const links = document.querySelectorAll('nav a');
    links.forEach(link => {{
        link.addEventListener('click', event => {{
            event.preventDefault();
            const href = link.getAttribute('href');
            history.pushState(null, null, href);
            handleRoute(); 
        }});
    }});
}});


window.addEventListener('popstate', handleRoute);

function handleLocation() {{
    handleRoute();
}}

const routes = {routes};

function action(route) {{
    history.pushState(null, null, route);
    handleLocation();
}}

handleLocation();
"""
print(Based)



class CodeBlock:
    def __init__(self, code: str, filename: str = None):
        self.code = code
        self.filename = filename

    def save_to_file(self):
        if self.filename:
            try:
                with open(f'./output/{self.filename}.js', 'w') as file:
                    file.write(self.code)
            except IOError as e:
                print(f"Error writing to file: {e}")
                return None
            return self.filename
        return None

    def get_code_tag(self):
        if self.code:
            return f'<script>{self.code}</script>'
        return None

    def __str__(self):
        return self.code



class Selectors:
    def __init__(self, ):
        self.id = None
        self.class_ = None
        self.query = None

    def IdSelector(self, id: str) ->str:
        return f'document.getElementById({self.id})'

    def ClassSelector(self, class_):
        self.class_ = class_

    def QuerySelector(self, query):
        self.query = query




class Image:
    def __init__(self, src, alt=None, **attributes):
        self.src = src
        self.alt = alt
        self.attributes = attributes

    def __str__(self):
        attribute_string = ' '.join([f'{key}="{value}"' for key, value in self.attributes.items()])
        alt_attribute = f'alt="{self.alt}"' if self.alt else ''
        return f'<img src="{self.src}" {alt_attribute} {attribute_string}>'

    def __add__(self, other):
        if isinstance(other, Image):
            return self.__str__() + other.__str__()
        else:
            return self.__str__() + str(other)



import re
from GooBa import Parent

class Router:
    def __init__(self):
        self.routes = {}
        self.dynamic_routes = []

    def render(self, route, content):
        if isinstance(content, Parent):
            content = str(content)  # Convert Parent to HTML string
        if '<' in route and '>' in route:
            # It's a dynamic route
            self.dynamic_routes.append((re.compile(route), content))
        else:
            # Escape quotes and newlines in content
            escaped_content = content.replace('"', '\\"').replace('\n', '\\n')
            self.routes[route] = f'"{escaped_content}"'

    def run(self, entry):
        # Prepare static routes
        route_cases = ", ".join(f'"{path}": {content}' for path, content in self.routes.items())

        # Prepare dynamic route handling
        dynamic_routes_js = ""
        for pattern, content in self.dynamic_routes:
            escaped_content = content.replace('"', '\\"').replace('\n', '\\n')
            dynamic_routes_js += f"""
                if (new RegExp({pattern.pattern}).test(path)) {{
                    html = `{escaped_content}`.replace(new RegExp({pattern.pattern}), path);
                }}
            """

        # Generate JavaScript code
        js_code = f"""
const route = (event) => {{
    event = event || window.event;
    event.preventDefault();
    window.history.pushState({{}}, "", event.target.href);
    handleLocation();
}};

const routes = {{
    {route_cases}
}};

const handleLocation = () => {{
    const path = window.location.pathname;
    let html = routes[path];

    if (!html) {{
        {dynamic_routes_js}
        if (!html) {{
            html = "<h2>404 Page Not Found</h2>";
        }}
    }}

    document.getElementById("{entry}").innerHTML = html;
}};

window.onpopstate = handleLocation;
window.route = route;

handleLocation();
"""
        with open('./output/router.js', 'w') as file:
            file.write(js_code)



from setuptools import setup, find_packages

setup(
    name="GooBa",
    version="0.1.4",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[],
    entry_points={
        'console_scripts': [
            # 'gooba-new=project_generator:create_project_structure',
        ],
    },
    author="Seth Linn Khant",
    author_email="sethlk2006@gmail.com",
    description="Gooba is a python frontend web framework inspired by Reactjs.",
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    url="https://github.com/SethLK/GooBa",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ],
    python_requires='>=3.6',
)



# Browser_Api
# like
# console.log
# alert
# windows.size
# ...etc

browser_Action = ""

def alert(text):
    browser_Action = f"alert('{text}')"
    return browser_Action

def run():
    with open('./output/actions.js', 'w') as file:
        file.write(browser_Action)


class Parent:
    def __init__(self, tag, **attributes):
        self.tag = tag
        self.attributes = attributes
        self.children = []

    def appendChild(self, *children):
        self.children.extend(children)

    def __str__(self):
        # Build attributes string, handle case where attributes might be empty
        if self.attributes:
            attribute_string = ' '.join(f'{key}="{value}"' for key, value in self.attributes.items())
            attributes_part = f' {attribute_string}'
        else:
            attributes_part = ''

        # Convert children to HTML, taking care of nested elements
        children_string = ''.join(str(child) for child in self.children)

        # Generate HTML string
        return f'<{self.tag}{attributes_part}>{children_string}</{self.tag}>'



class Element:
    def __init__(self, tag, **attributes):
        self.tag = tag
        self.attributes = attributes
        self.text = None

    # def set_text(self, text):
    #     self.text = text

    def __str__(self):
        attribute_string = ' '.join([f'{key}="{value}"' for key, value in self.attributes.items()])
        if self.text is not None:
            return f'<{self.tag} {attribute_string}>{self.text}</{self.tag}>'
        else:
            return f'<{self.tag} {attribute_string}></{self.tag}>'

    def __add__(self, other):
        if isinstance(other, Element):
            return self.__str__() + other.__str__()
        else:
            return self.__str__() + str(other)


class Image:
    def __init__(self, src, alt=None, **attributes):
        self.src = src
        self.alt = alt
        self.attributes = attributes

    def __str__(self):
        attribute_string = ' '.join([f'{key}="{value}"' for key, value in self.attributes.items()])
        alt_attribute = f'alt="{self.alt}"' if self.alt else ''
        return f'<img src="{self.src}" {alt_attribute} {attribute_string}>'

    def __add__(self, other):
        if isinstance(other, Image):
            return self.__str__() + other.__str__()
        else:
            return self.__str__() + str(other)



javascript = """
            // Function to reload the page
            function reloadPage() {
                window.location.reload();
            }

            // Function to watch for changes
            function watchForChanges() {
                var xhr = new XMLHttpRequest();
                xhr.onreadystatechange = function() {
                    if (xhr.readyState === XMLHttpRequest.DONE) {
                        if (xhr.status === 200) {
                            // Check if the response has changed
                            if (xhr.responseText !== localStorage.getItem('lastResponse')) {
                                localStorage.setItem('lastResponse', xhr.responseText);
                                reloadPage();
                            }
                        }
                    }
                }
                xhr.open('GET', window.location.href + '?reload=' + new Date().getTime(), true);
                xhr.send();
            }

            // Call the watchForChanges function periodically
            setInterval(watchForChanges, 1000);

        """